@@ -22,9 +22,9 @@
 
 #include <config.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <gtk/gtk.h>
-#include <gnome.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -36,7 +36,7 @@
 #include "gtk-utils.h"
 
 
-#define GLADE_FILE "file_roller.glade"
+#define GLADE_FILE "file-roller.glade"
 
 
 typedef struct {
@@ -47,36 +47,34 @@ typedef struct {
 
 
 static void
-open_file_destroy_cb (GtkWidget *w,
-		      GtkWidget *file_sel)
+open_file_destroy_cb (GtkWidget  *file_sel,
+		      DialogData *data)
 {
-	DialogData *data;
-	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
 	g_free (data);
 }
 
 
 static int
-file_sel_response_cb (GtkWidget      *w,
+file_sel_response_cb (GtkWidget      *widget,
 		      int             response,
-		      GtkFileChooser *file_sel)
+		      DialogData     *data)
 {
-	FRWindow   *window;
-	char       *current_folder;
-	gboolean    update;
-	GSList     *selections, *iter;
-	GList      *item_list = NULL;
-	DialogData *data;
+	GtkFileChooser *file_sel = GTK_FILE_CHOOSER (widget);
+	FRWindow       *window = data->window;
+	char           *current_folder;
+	gboolean        update;
+	GSList         *selections, *iter;
+	GList          *item_list = NULL;
 
-
-	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
-
-	if (response == GTK_RESPONSE_CANCEL) {
+	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (data->dialog);
 		return TRUE;
 	}
 
-	window = data->window;
+	if (response == GTK_RESPONSE_HELP) {
+		show_help_dialog (GTK_WINDOW (data->dialog), "file-roller-add-options");
+		return TRUE;
+	}
 
 	current_folder = gtk_file_chooser_get_current_folder (file_sel);
 
@@ -88,7 +86,7 @@ file_sel_response_cb (GtkWidget      *w,
 		char      *utf8_path;
 		char      *message;
 
-		utf8_path = g_filename_to_utf8 (current_folder, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (current_folder);
 		message = g_strdup_printf (_("You don't have the right permissions to read files from folder \"%s\""), utf8_path);
 		g_free (utf8_path);
 
@@ -96,14 +94,11 @@ file_sel_response_cb (GtkWidget      *w,
 					     GTK_DIALOG_MODAL,
 					     GTK_STOCK_DIALOG_ERROR,
 					     _("Could not add the files to the archive"),
-					     _("You don't have the right permissions to read files from folder \"%s\""),
-					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_OK, GTK_RESPONSE_OK,
+					     message,
+					     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 					     NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
-		g_free (utf8_path);
 
 		g_free (current_folder);
 
@@ -123,7 +118,7 @@ file_sel_response_cb (GtkWidget      *w,
 	}
 
 	if (item_list != NULL) 
-		window_archive_add_items (window, item_list, update);
+		window_archive_add_dropped_items (window, item_list, update);
 
 	g_list_free (item_list);
 	g_slist_foreach (selections, (GFunc) g_free, NULL);
@@ -136,6 +131,43 @@ file_sel_response_cb (GtkWidget      *w,
 }
 
 
+static void
+selection_changed_cb (GtkWidget  *file_sel, 
+ 		      DialogData *data)
+{
+	FRWindow   *window = data->window;
+	char       *current_folder;
+
+        current_folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (file_sel));
+
+        /* check folder permissions. */
+                                                                                                
+        if (path_is_dir (current_folder)
+            && access (current_folder, R_OK | X_OK) != 0) {
+                GtkWidget *d;
+                char      *utf8_path;
+                char      *message;
+		
+                utf8_path = g_filename_display_name (current_folder);
+                message = g_strdup_printf (_("You don't have the right permissions to read files from folder \"%s\""), utf8_path);
+                g_free (utf8_path);
+		
+                d = _gtk_message_dialog_new (GTK_WINDOW (window->app),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_STOCK_DIALOG_ERROR,
+                                             _("Could not add the files to the archive"),
+                                             message,
+                                             GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+                                             NULL);
+                gtk_dialog_run (GTK_DIALOG (d));
+                gtk_widget_destroy (GTK_WIDGET (d));
+                g_free (message);
+		
+                g_free (current_folder);
+        }
+}
+
+
 /* create the "add" dialog. */
 void
 add_files_cb (GtkWidget *widget, 
@@ -143,11 +175,8 @@ add_files_cb (GtkWidget *widget,
 {
 	GtkWidget   *file_sel;
 	DialogData  *data;
-	gchar       *dir;
+	char        *dir;
 	GtkWidget   *main_box;
-	GtkTooltips *tooltips;
-	
-	tooltips = gtk_tooltips_new ();
  
 	data = g_new0 (DialogData, 1);
 	data->window = callback_data;
@@ -155,10 +184,9 @@ add_files_cb (GtkWidget *widget,
 		gtk_file_chooser_dialog_new (_("Add Files"),
 					     GTK_WINDOW (data->window->app),
 					     GTK_FILE_CHOOSER_ACTION_OPEN,
-					     GTK_STOCK_CANCEL, 
-					     GTK_RESPONSE_CANCEL,
-					     FR_STOCK_ADD, 
-					     GTK_RESPONSE_OK,
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     FR_STOCK_ADD, GTK_RESPONSE_OK,
+					     GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 					     NULL);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_sel), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
@@ -181,20 +209,23 @@ add_files_cb (GtkWidget *widget,
 	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_sel), dir);
 	g_free (dir);
 
-	g_object_set_data (G_OBJECT (file_sel), "fr_dialog_data", data);
-
 	/* signals */
 	
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (open_file_destroy_cb),
-			  file_sel);
+			  data);
 
 	g_signal_connect (G_OBJECT (file_sel),
 			  "response",
 			  G_CALLBACK (file_sel_response_cb),
-			  file_sel);
+			  data);
 
-	gtk_window_set_modal (GTK_WINDOW (file_sel),TRUE);
+	g_signal_connect (G_OBJECT (file_sel), 
+			  "selection-changed",
+			  G_CALLBACK (selection_changed_cb),
+			  data);
+
+	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 	gtk_widget_show (file_sel);
 }
